/*
 * frilogos.c
 * This file is part of FriLogos
 *
 * Copyright (C) 2012 - Alexandre Dussart
 *
 * FriLogos is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * FriLogos is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FriLogos. If not, see <http://www.gnu.org/licenses/>.
 */


#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <time.h>

#include <Ecore_File.h>

#include "frilogos.h"

EAPI int elm_main(int argc, char *argv[]);

int _log_dom;


//Widgets callbacks.
static void quit_bt_clicked_cb(void *data __UNUSED__, Evas_Object *obj __UNUSED__, void *event_info __UNUSED__);
static void _modules_list_selected_cb(void *data, Evas_Object *obj, void *event_info);
static void _patients_list_selected_cb(void *data, Evas_Object *obj, void *event_info);
static void _fill_modules_list(App_Info *app);


static void _patients_list_selected_cb(void *data, Evas_Object *obj, void *event_info)
{
	char s[128];
	App_Info *app = data;
	Elm_Object_Item *it = event_info;


	app->patient_selection = elm_object_item_data_get(it);
	
    if(eina_list_count(elm_list_items_get(app->modules_list)) == 0)
        _fill_modules_list(app);
        	    
        if(app->module_selection)    	    
        	app->module_selection->_module_patient_set(app->patient_selection);
	        	    
	snprintf(s, sizeof(s), "%s %s", app->patient_selection->firstname, app->patient_selection->lastname);

	elm_object_text_set(obj, elm_entry_markup_to_utf8(s));
}


static void _modules_list_selected_cb(void *data, Evas_Object *obj __UNUSED__, void *event_info)
{
	App_Info *app = data;
	Module_Info *module;
    Eina_List *its, *l;
    Elm_Object_Item *it;

	module = elm_object_item_data_get(event_info);
	app->module_selection = module;
	elm_entry_entry_set(app->description_text, module->description);
    
    its = elm_naviframe_items_get(app->naviframe);
    EINA_LIST_FOREACH(its, l, it)
    {
        char *s = elm_object_item_data_get(it);
        if(strcmp(s, module->id) == 0)
        {
            elm_naviframe_item_promote(it);
            break;
        }
    }
    
    if(app->patient_selection)    	    
        app->module_selection->_module_patient_set(app->patient_selection);    
}


static void quit_bt_clicked_cb(void *data __UNUSED__, Evas_Object *obj __UNUSED__, void *event_info __UNUSED__)
{
   elm_exit();
}


void _fill_modules_list(App_Info *app)
{
    Eina_List *modules, *l;
	Module_Info *module;
	
	modules = modules_list_get(app->win);

    EINA_LIST_FOREACH(modules, l, module)
	{
		Evas_Object *ic;
		ic=elm_icon_add(app->win);
		elm_icon_file_set(ic, module->icon, "icon");
		elm_icon_resizable_set(ic, EINA_FALSE, EINA_FALSE);
		evas_object_show(ic);
		elm_list_item_append(app->modules_list, module->name, ic, ic, NULL, module);
		
        Elm_Object_Item *it = elm_naviframe_item_push(app->naviframe, _("Options"), NULL, NULL, module->settings, NULL);
        elm_naviframe_item_title_visible_set(it, EINA_FALSE);            
        elm_object_item_data_set(it, module->id);
	}
		
	//Push a new empty item to naviframe to set when no module is selected(at frilogos startup).
	Elm_Object_Item *it = elm_naviframe_item_push(app->naviframe, _("Options"), NULL, NULL, NULL, NULL);
	elm_naviframe_item_title_visible_set(it, EINA_FALSE);            
    elm_object_item_data_set(it, "");	
	elm_list_go(app->modules_list);
	evas_object_data_set(app->win, "modules", modules);    
}


EAPI int elm_main(int argc, char **argv)
{
	char s[512];
	time_t timestamp;
	struct tm *t;
	Evas_Object *vbx, *vbx2, *bg, *frame;
	Evas_Object *hs;
	Evas_Object *sep;
	App_Info *app;
            
            
	// Initialize important stuff like eina debug system, ecore_evas, eet, elementary...
	#if ENABLE_NLS
	setlocale(LC_ALL, "");
	bindtextdomain(PACKAGE_NAME, frilogos_locale_path_get());
	bind_textdomain_codeset(PACKAGE_NAME, "UTF-8");
	textdomain(PACKAGE_NAME);
	#endif


    fprintf(stdout, _("INF:%s frilogos.c: 118 elm_main() Initialize EFL...\n"), PACKAGE_NAME);

	if (!eina_init())
	{
		fprintf(stderr, _("CRI:Unable to init Eina.\n"));
               	return EXIT_FAILURE;
	}

    _log_dom = eina_log_domain_register("frilogos", EINA_COLOR_CYAN);
    if(_log_dom < 0)
    {
        ERR(_("Enable to create a log domain."));
        exit(1);
    }
    eina_log_domain_level_set("frilogos", 3);
    eina_log_color_disable_set(EINA_TRUE);

	eet_init();
	ecore_init();
	ecore_evas_init();
	edje_init();

/*
	#ifdef HAVE_EVIL
    elm_engine_set("software_gdi");
	#else
    elm_engine_set("software_x11");
	#endif
*/
	if (!elm_init(argc, argv))
	{
		CRITICAL(_("Can't init Elementary."));
               	return EXIT_FAILURE;
	}

	INF(_("Initialize Application..."));
	app = malloc(sizeof(*app));
	app->patient_selection = NULL;
	app->module_selection = NULL;

    //Init FriLogos.
	frilogos_init();
	//ConteneurMalloc_Affiche (EINA_TRUE);

	// Initialize random generator to avoid reccurent values.
	SeedRandomizer();

	//Setup main window.
	timestamp = time(NULL);
	t = localtime(&timestamp);

	snprintf(s, sizeof(s), _("FriLogos %s - Session of %02d/%02d/%d"),
	   				PACKAGE_VERSION,
	   				(int)t->tm_mday,
	  				(int)t->tm_mon+1,
	  				1900+(int)t->tm_year);

	app->win = elm_win_add(NULL, "FriLogos", ELM_WIN_BASIC);
	elm_win_title_set(app->win, s);
	elm_win_autodel_set(app->win, EINA_TRUE);
	evas_object_show(app->win);
    elm_win_center(app->win, EINA_TRUE, EINA_TRUE);
	evas_object_data_set(app->win, "App_Info", app);

  	bg = elm_bg_add(app->win);
	evas_object_size_hint_weight_set(bg, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	elm_win_resize_object_add(app->win, bg );
	evas_object_show(bg );
  	evas_object_size_hint_min_set(bg, 850, 720);
  	
	vbx = elm_box_add(app->win);
	evas_object_size_hint_weight_set(vbx, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	elm_win_resize_object_add(app->win, vbx);
	evas_object_show(vbx);

	//Setup toolbar.
	app->toolbar = elm_toolbar_add(app->win);
	elm_toolbar_icon_order_lookup_set(app->toolbar, ELM_ICON_LOOKUP_FDO_THEME);
	evas_object_size_hint_align_set(app->toolbar, -1.0, 0.0);
	evas_object_size_hint_weight_set(app->toolbar, 1.0, 0.0);

	elm_toolbar_item_append(app->toolbar,"patients-browser", _("Patients"), patients_browser_new, app);
	elm_toolbar_item_append(app->toolbar, "items-browser", _("Items"), items_browser_new, app);
	elm_toolbar_item_append(app->toolbar,"about-dlg", _("About"), about_dialog_new, app);
	elm_toolbar_item_append(app->toolbar, "window-close", _("Quit"), quit_bt_clicked_cb, app);
	elm_box_pack_end(vbx, app->toolbar);
	evas_object_show(app->toolbar);

	Evas_Object *ck;
	ck = elm_clock_add(app->win);
	elm_clock_edit_set(ck, EINA_FALSE);
	elm_clock_show_seconds_set(ck, EINA_TRUE);
	elm_clock_show_am_pm_set(ck, EINA_TRUE);
  	evas_object_size_hint_max_set(ck, 250, 70);
	elm_box_pack_end(vbx, ck);
	evas_object_show(ck);

	hs = elm_hoversel_add(app->win);
	evas_object_data_set(app->win, "PatientsHoverSel", hs);
	elm_hoversel_hover_parent_set(hs, app->win);
	elm_object_text_set(hs, _("Patient selection"));
	evas_object_size_hint_align_set(hs, 0.5, 0.5);
	elm_box_pack_end(vbx, hs);
	evas_object_show(hs);

    //Read patients files and fill hoversel widget.
	Eina_List *patients = NULL;
	Patient_Info *patient;
	Eina_List *l;
	patients = patients_list_get();
	EINA_LIST_FOREACH(patients, l, patient)
	{
		snprintf(s, sizeof(s), "%s %s",elm_entry_markup_to_utf8(patient->firstname), elm_entry_markup_to_utf8(patient->lastname));
		elm_hoversel_item_add(hs, s, NULL, ELM_ICON_NONE, NULL, patient);
	}
	evas_object_smart_callback_add(hs, "selected", _patients_list_selected_cb, app);

	sep = elm_separator_add(app->win);
	elm_separator_horizontal_set(sep, EINA_TRUE);
	elm_box_pack_end(vbx, sep);
	evas_object_show(sep);

	//Create module list panel selector.
    Evas_Object *panes = elm_panes_add(app->win);
    elm_win_resize_object_add(app->win, panes);
    evas_object_size_hint_weight_set(panes, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
    evas_object_size_hint_align_set(panes, EVAS_HINT_FILL, EVAS_HINT_FILL);
	elm_box_pack_end(vbx, panes);
    evas_object_show(panes);
   
	app->modules_list = elm_list_add(app->win);
	elm_list_select_mode_set(app->modules_list ,ELM_OBJECT_SELECT_MODE_ALWAYS );
	evas_object_size_hint_weight_set(app->modules_list, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	evas_object_size_hint_align_set(app->modules_list, -1.0, -1.0);
	evas_object_smart_callback_add(app->modules_list, "selected", _modules_list_selected_cb, app);
	evas_object_show(app->modules_list);
	elm_object_part_content_set(panes, "left", app->modules_list);
    elm_panes_content_left_size_set(panes, 0.25);
	   
	vbx2 = elm_box_add(app->win);
	evas_object_size_hint_weight_set(vbx2, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	evas_object_size_hint_align_set(vbx2, -1.0, -1.0);
	evas_object_show(vbx2);

	frame = elm_frame_add(app->win);
	evas_object_size_hint_align_set(frame, -1.0, -1.0);

	elm_object_text_set(frame, _("Description"));
	elm_box_pack_end(vbx2, frame);
	evas_object_show(frame);

  	app->description_text =  elm_entry_add(app->win);
	elm_entry_line_wrap_set(app->description_text, EINA_TRUE);
	elm_entry_editable_set(app->description_text, EINA_FALSE);
   	elm_entry_entry_set(app->description_text, _("<b>FriLogos</>Welcome to FriLogos!<br><br>"
   	                                              "You should select or create a patient to start a session with him."));
	evas_object_size_hint_weight_set(app->description_text, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	elm_object_content_set(frame, app->description_text);
	evas_object_show(app->description_text);

	app->naviframe = elm_naviframe_add(app->win);
	evas_object_size_hint_weight_set(app->naviframe, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	evas_object_size_hint_align_set(app->naviframe, -1.0, -1.0);
	elm_object_text_set(app->naviframe, _("Options"));
	elm_box_pack_end(vbx2, app->naviframe);	
	evas_object_show(app->naviframe);
	elm_object_part_content_set(panes, "right", vbx2);


	app->patient_selection = patient_read("/home/alex/default.eet");
    _fill_modules_list(app);


    //items_list_debug_test();
   // patients_list_debug_test();

    //char *ret;
    //settings_write("wanttoolbar", "no");
    //ret = settings_read("wanttoolbar");
    //INF("Test wanttoolbar setting:%s", ret);

   // Patient_Info *p;
   //p = patient_new();
   //patient_write(p);
   //patient_remove(p);
   //p = patient_free(p);
  	
	elm_run();

	INF(_("Shutdown FriLogos..."));
	app->patient_selection = patient_free(app->patient_selection);
    app->module_selection = module_free(app->module_selection);
	patients = patients_list_free(patients);
	Eina_List *modules = evas_object_data_get(app->win, "modules");    
    modules = modules_list_free(modules);
	modules = evas_object_data_del(app->win, "modules");    
	free(app);

	eina_shutdown();
	ecore_evas_shutdown();
	ecore_shutdown();
	eet_shutdown();
	edje_shutdown();
	elm_shutdown();
	exit(0);

	return EXIT_SUCCESS;
}
ELM_MAIN()
