/*
 * halloc.c
 * This file is part of FriLogos
 *
 * Modified Copyright (C) 2012 - Alexandre Dussart
 * Derivated from HAlloc project HaypoCALC by Haypo (http://www.haypocalc.com/).
 *
 * FriLogos is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * FriLogos is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FriLogos. If not, see <http://www.gnu.org/licenses/>.
 */


#include "halloc.h"


#ifdef DEBUG_MALLOC_CONTENEUR
  const char *delete_FILE = NULL;
  unsigned long delete_LINE = 0;
#endif

#ifdef DEBUG_MALLOC_CONTENEUR
  // Le conteneur Malloc
  struct
  {
    struct PtrChaineMalloc *debut;   // Pointeur vers le premier lment
    struct PtrChaineMalloc *fin;     // Pointeur vers le dernier lment
  } ConteneurMalloc = {NULL,NULL};
  const unsigned long TAILLE_INFO_MALLOC=sizeof(InfoMalloc);
#else
  const unsigned long TAILLE_INFO_MALLOC=0;
#endif

#ifdef DEBUG_MALLOC_OVERFLOW
  const unsigned long TAILLE_TAMPON_OVERFLOW=16;
#else
  const unsigned long TAILLE_TAMPON_OVERFLOW=0;
#endif


//
//Disable macros to secure memory allocation.
//
#undef malloc
#undef calloc
#undef realloc
#undef strdup
#undef free
#undef new
#undef delete

//Magic numbers to valid pointer.
#define MALLOC_NBR_MAGIQUE 0x1A71A25C

//
//Anti-overflow(writing over allocated zone) tests.
//
#ifdef DEBUG_MALLOC_OVERFLOW
// Prpare la zone anti-overflow d'un pointeur
// (ptr pointe sur les infos de dbogage InfoMalloc)
void Malloc_PrepareOverflow (InfoMalloc *info)
{
  char *ptr;
  unsigned long i;
  unsigned char rand = 0xA0;

  // Calcule la position du tampon anti-overflow
  ptr = (char *)info +sizeof(InfoMalloc) +info -> taille;

  // Remplit le tampon avec des valeurs fixes
  for (i=0; i<TAILLE_TAMPON_OVERFLOW; i++)
  {
    *ptr = rand;
    ptr++;
    rand++;
  }
}


//Check anti-overflow buffer.
//Return true if buffer is ok.
//(ptr pointe sur les informations de dbogage InfoMalloc)
Eina_Bool Malloc_TamponOverflowIntact (const InfoMalloc *info)
{
  char *ptr;
  unsigned long i;
  unsigned char rand = 0xA0;

  // Calcule la position du tampon anti-overflow
  ptr = (char *)info +sizeof(InfoMalloc) +info -> taille;

  // Remplit le tampon avec des valeurs fixes
  for (i=0; i<TAILLE_TAMPON_OVERFLOW; i++)
  {
    // Valeur diffrente de celle dfinie ?
    if ((unsigned char)*ptr != rand)
    {
      // Erreur !
      return EINA_FALSE;
    }
    ptr++;
    rand++;
  }

  // Tampon intact
  return EINA_TRUE;
}
#endif

//
//Conteneur Malloc : liste des pointeurs allous.
//
#ifdef DEBUG_MALLOC_CONTENEUR
// Calcule la somme de vrification (checksum) des donnes d'un pointeur
unsigned long MallocCalculeSomme (const InfoMalloc *info)
{
  // Calcule la somme des donnes d'un pointeur
  return
   ((unsigned long)(info -> nbr_magique)
   +(unsigned long)(info -> est_alloue)
   +(unsigned long)(info -> taille)
   +(unsigned long)(info -> nom_fich)
   +(unsigned long)(info -> num_ligne));
}

//
// Fonction qui crit les informations d'un pointeur et modifie son adresse
// pour qu'il pointe sur les donnes
void *MallocPreparePtr (InfoMalloc *info, size_t size,
                        const char *nom_fich, unsigned long num_ligne)
{
  //Malloc_Info header.
  info -> nbr_magique = MALLOC_NBR_MAGIQUE;
  info -> est_alloue = EINA_TRUE;
  info -> taille = size;
  info -> nom_fich = nom_fich;
  info -> num_ligne = num_ligne;
  info -> somme_info = MallocCalculeSomme (info);

#ifdef DEBUG_MALLOC_OVERFLOW
  //Prpare le tampon anti-overflow.
  Malloc_PrepareOverflow (info);
#endif

  //Passe les informations pour obtenir un pointeur sur les donnes.
  return (info+1);
}

//
// Ajoute un pointeur au conteneur malloc
//
void ConteneurMalloc_Ajoute (InfoMalloc* info)
{
  // Tente d'allouer un nouvel lment  la liste
  struct PtrChaineMalloc *elem;
  elem = (struct PtrChaineMalloc *)malloc(sizeof(struct PtrChaineMalloc));

  // Plus de mmoire ):-(
  if (elem == NULL)
  {
    // Erreur
    CRITICAL (
      "Impossible d'agrandir le conteneur des malloc : plus assez "
      "de mmoire libre disponnible !");
  }

  // Sauve les informations
  elem -> nbr_magique = MALLOC_NBR_MAGIQUE;
  elem -> ptr_info = info;
  info -> ptr_liste = elem;

  //--- Ajout en queue ---

  // Modifie le dbut de la liste (si ncessaire)
  if (ConteneurMalloc.debut == NULL) ConteneurMalloc.debut = elem;

  // Ajoute l'lment en queue
  elem -> suivant = NULL;
  if (ConteneurMalloc.fin != NULL) ConteneurMalloc.fin -> suivant = elem;
  elem -> precedent = ConteneurMalloc.fin;
  ConteneurMalloc.fin = elem;
}


//
//Vrifie les informations sur un pointeur.
//
void VerifieInfoMalloc (InfoMalloc* info)
{
  // Vrifie les nombres magiques
  if (info -> nbr_magique != MALLOC_NBR_MAGIQUE)
    CRITICAL ("Pointeur invalide");

  // Vrifie la somme
  if (info -> somme_info != MallocCalculeSomme (info))
    CRITICAL ("Somme incorrecte");

#ifdef DEBUG_MALLOC_OVERFLOW
  // Vrifie qu'on est pas crit en dehors du tampon
  if (!Malloc_TamponOverflowIntact(info))
  {
    CRITICAL ("Erreur d'overflow (tampon %s:%lu)",
            info -> nom_fich, info -> num_ligne);
  }
#endif
}


//
// Affiche le contenu du conteneur malloc (libre les pointeurs si
// libere_mem=true)
void ConteneurMalloc_Affiche (Eina_Bool libere_mem)
{
  struct PtrChaineMalloc *elem, *suivant;
  Eina_Bool titre_affiche = EINA_FALSE;  // Le titre a dj t affich ?
  unsigned long nbr_pointeur=0;       // Nombre de pointeurs non librs
  unsigned long taille_totale=0;      // Taille totale en octet non libre
  InfoMalloc* info;
  char* vrai_ptr;

  // Parcoure la liste du dbut  la fin
  elem = ConteneurMalloc.debut;
  while (elem != NULL)
  {
    // Sauve le pointeur suivant
    suivant = elem -> suivant;

    // Calcule l'adresse du vrai pointeur
    info = elem -> ptr_info;
    vrai_ptr = (char *)(info+1);

    // Vrifie les informations
    VerifieInfoMalloc (info);

    // Affiche le titre si ncessaire
    if (titre_affiche == EINA_FALSE)
    {
      titre_affiche = EINA_TRUE;
      printf ("[ Memoire non liberee ]\n");
    }

    // Affiche le nom du fichier et la ligne
    printf ("%s:%lu - ",
            info -> nom_fich,
            (unsigned long)info -> num_ligne);

    // Affiche le pointeur si demand
    printf ("pointeur %p de taille %lu\n",
            vrai_ptr,
            (unsigned long)info -> taille);

    // Ajoute tout ca au total
    nbr_pointeur++;
    taille_totale += info -> taille;

    // Sort le pointeur de la liste
    if (libere_mem)
      ConteneurMalloc_Efface (elem);

    // Passe au pointeur suivant
    elem = suivant;
  }

  // Affiche le total
  if (titre_affiche == EINA_TRUE)
    printf ("TOTAL = %lu pointeur(s) pour %lu octets.", nbr_pointeur, taille_totale);
}

//
// Sort un pointeur du conteneur malloc.
//
void ConteneurMalloc_Remplace (InfoMalloc *info, InfoMalloc *nv_info)
{
  struct PtrChaineMalloc *elem;

  // Cherche les informations
  elem = info -> ptr_liste;
  if (elem -> nbr_magique != MALLOC_NBR_MAGIQUE)
    CRITICAL ("Erreur de rallocation de mmoire");

  // Enfin, remplace le pointeur
  elem -> ptr_info = nv_info;
}

//
// Sort un pointeur du conteneur malloc.
//
void ConteneurMalloc_Efface (struct PtrChaineMalloc* elem)
{
  // Pointeur dj dsallou ?
  if (elem -> ptr_info -> est_alloue == EINA_FALSE)
  {
    CRITICAL ("Pointeur dj desallou !");
  }

  // Sort l'lment de la liste
  if (elem -> precedent != NULL)
    (elem -> precedent) -> suivant = elem -> suivant;
  if (elem -> suivant != NULL)
    (elem -> suivant) -> precedent = elem -> precedent;

  // Change le dbut (si l'lment tait au dbut de la liste)
  if (ConteneurMalloc.debut == elem) ConteneurMalloc.debut = elem -> suivant;

  // Change la fin (si l'lment tait  la fin de la liste)
  if (ConteneurMalloc.fin == elem) ConteneurMalloc.fin = elem -> precedent;

  // Dtruit l'lment (ne libre pas la mmoire)
  free (elem);
}

//
// Securized free.
// free scuris : utilis pour le conteneur
void FreeSecurise (const void *ptr)
{
  InfoMalloc *Info;

  // Pointeur NULL : La norme demande de ne rien faire, on va pas se gner!
  if (ptr == NULL) return;

  // Rgle le pointeur sur les donnes
  Info = (InfoMalloc *)ptr-1;

  // Vrifie les informations
  VerifieInfoMalloc (Info);

  // Sort le pointeur du conteneur en vrifiant les
  // informations sur le pointeur
  ConteneurMalloc_Efface (Info -> ptr_liste);

  // Le pointeur n'est plus allou !
  Info -> est_alloue = EINA_FALSE;

  // Enfin, libre la mmoire
  free (Info);
}
#endif // de "#ifdef DEBUG_MALLOC_CONTENEUR"


//
//Allocation scurise de mmoire.
//
#ifdef DEBUG_MALLOC
// Calcule la taille relle d'un bloc allou
size_t MallocCalculeTaille (size_t size)
{
 return size +(TAILLE_INFO_MALLOC+TAILLE_TAMPON_OVERFLOW);
}

//-----------------------------------------------------------------------------

// Allocation de mmoire scurise (malloc) : test si malloc renvoie NULL
void* MallocSecurise (size_t size,
                      const char* nom_fich, const unsigned long num_ligne)
{
  // Tente d'allouer le tampon
  void* ptr;
  size_t TailleAllouee;

  // Taille nulle : Erreur!
  if (size == 0)
    CRITICAL ("Taille du pointeur  allouer nulle !");

  // Ajoute la taille des informations sur le pointeur
  TailleAllouee = MallocCalculeTaille (size);

  // Alloue la mmoire
  ptr = malloc(TailleAllouee);

  // Pointeur NULL : erreur!
  if (ptr == NULL)
    CRITICAL ("Allocation de mmoire impossible :pas assez de mmoire libre");

#ifdef DEBUG_MALLOC_CONTENEUR
  // Ajoute le pointeur au conteneur
  ConteneurMalloc_Ajoute ((InfoMalloc *)ptr);

  // Ecrit les informations sur le pointeur et modifie l'adresse
  // du pointeur (si on compile avec le conteneur malloc)
  ptr = MallocPreparePtr ((InfoMalloc *)ptr,size,nom_fich,num_ligne);
#endif

  // Retourne le pointeur
  return ptr;
}

//-----------------------------------------------------------------------------

// calloc scuris : test si calloc renvoie NULL
void* CallocSecurise (size_t nitems, size_t size,
                      const char* nom_fich, const unsigned long num_ligne)
{
  void* ptr;

  // Alloue de la mmoire
  ptr = MallocSecurise (size*nitems,nom_fich,num_ligne);

  // Mise  zro des octets (fonctionnalit de 'calloc'=malloc+memset)
  memset (ptr,0,size);

  // Retourne le pointeur
  return ptr;
}

//-----------------------------------------------------------------------------

// realloc scuris : vrifie que l'allocation se soit bien produite,
// affiche une erreur sinon
void* ReallocSecurise (void* ptr, size_t size,
                       const char* nom_fich, const unsigned long num_ligne)
{
  void* Nouveau;
#ifdef DEBUG_MALLOC_CONTENEUR
  InfoMalloc *NvInfo, *Info;
#endif
  size_t TailleAllouee;

  // Si l'ancien pointeur actuel vaut NULL : appelle malloc
  if (ptr == NULL) return MallocSecurise (size, nom_fich, num_ligne);

  // Ajoute la taille des informations sur le pointeur
  TailleAllouee = MallocCalculeTaille (size);

#ifdef DEBUG_MALLOC_CONTENEUR
  // Positionne le pointeur sur les donnes
  Info = (InfoMalloc *)ptr-1;

  // Vrifie le pointeur
  VerifieInfoMalloc (Info);

  // Tente d'agrandir le pointeur
  Nouveau = realloc(Info,TailleAllouee);
#else
  // Tente d'agrandir le pointeur
  Nouveau = realloc(ptr,TailleAllouee);
#endif

  // Rallocation impossible ?
  if (Nouveau == NULL)
    CRITICAL ("Allocation de mmoire impossible : "
            "pas assez de mmoire libre");

#ifdef DEBUG_MALLOC_CONTENEUR
  // Remplace l'ancien pointeur par le nouveau dans le conteneur
  NvInfo = (InfoMalloc *)Nouveau;
  ConteneurMalloc_Remplace (Info,NvInfo);

  // Ecrit les informations sur le pointeur
  Nouveau = MallocPreparePtr (NvInfo,size,nom_fich,num_ligne);
#endif

  // Renvoie le pointeur
  return Nouveau;
}

//-----------------------------------------------------------------------------

// Cre une copie d'une chane avec test si l'allocation renvoie NULL
char* StrdupSecurise(const char* src, const char* nom_fich, const unsigned long num_ligne)
{
  // Tente de copier la chaine
  char* ptr;

#ifdef DEBUG_MALLOC_CONTENEUR
  // Alloue la mmoire
  ptr = (char *)MallocSecurise (strlen(src)+1, nom_fich, num_ligne);
#else
  ptr = (char *)malloc (strlen(src)+1);
  if (ptr == NULL)
    CRITICAL ("Impossible de dupliquer une chane de caractre : plus assez de mmoire disponnible !");
#endif

  // Copie la chaine
  strcpy (ptr,src);

  // Retourne le pointeur
  return ptr;
}

//-----------------------------------------------------------------------------
//--- Allocation C++ : new/delete ---------------------------------------------
//-----------------------------------------------------------------------------

#ifdef __cplusplus
// Surchage de l'oprateur new
void* operator new (size_t size, const char* nom, const unsigned long ligne)
  throw (std::bad_alloc)
{
  return MallocSecurise (size, nom, ligne);
}

//-----------------------------------------------------------------------------

// Surchage de l'oprateur new[]
void* operator new[] (size_t size, const char* nom, const unsigned long ligne)
  throw (std::bad_alloc)
{
  return MallocSecurise (size, nom, ligne);
}

//-----------------------------------------------------------------------------

// Surchage de l'oprateur delete
void operator delete (void *ptr) throw()
{
#ifdef DEBUG_MALLOC_CONTENEUR
  FreeSecurise (ptr, delete_FILE, delete_LINE);
  delete_FILE = NULL;
  delete_LINE = 0;
#else
  free (ptr);
#endif
}

//-----------------------------------------------------------------------------

// Surchage de l'oprateur delete[]
void operator delete[] (void *ptr) throw()
{
#ifdef DEBUG_MALLOC_CONTENEUR
  FreeSecurise (ptr, delete_FILE, delete_LINE);
  delete_FILE = NULL;
  delete_LINE = 0;
#else
  free (ptr);
#endif
}
#endif // de "#ifdef __cplusplus"
#endif // de "#ifdef DEBUG_MALLOC"

//-----------------------------------------------------------------------------
