/*
 * libfrilogos.h
 * This file is part of FriLogos
 *
 * Copyright (C) 2012 - Alexandre Dussart
 *
 * FriLogos is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * FriLogos is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FriLogos. If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef __LIBFRILOGOS_H__
#define __LIBFRILOGOS_H__

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <Eina.h>
#undef EINA_LOG_DOMAIN_DEFAULT
#define EINA_LOG_DOMAIN_DEFAULT _log_dom
extern int _log_dom;

#include <Evas.h>

#include "gettext.h"

#define _(String) gettext (String)
#define gettext_noop(String) String
#define N_(String) gettext_noop (String)

#define __UNUSED__ __attribute__((unused))

#define CRITICAL(...) EINA_LOG_CRIT( __VA_ARGS__)
#define ERR(...) EINA_LOG_ERR( __VA_ARGS__)
#define WRN(...) EINA_LOG_WARN(__VA_ARGS__)
#define INF(...) EINA_LOG_INFO(__VA_ARGS__)
#define DBG(...) EINA_LOG_DBG( __VA_ARGS__)

#ifdef HAVE_EVIL
#define DIR_SEPARATOR '\\'
#define DIR_SEPARATOR_S "\\"
#else
#define DIR_SEPARATOR '/'
#define DIR_SEPARATOR_S "/"
#endif


//Define groups infos.
typedef struct
{
	int id;
	char *name;
  	Eina_List *subgroups;
  	//Eina_List *Items;
} Group_Info;

//Define distractor_type.
typedef enum
{
    UNDEFINED,
    NEUTRAL,
    GRAPHEMIC,
    VISUAL,
    SEMANTIC
} Distractor_Type;


//Define Item Infos.
typedef struct
{
    char *version;
   	char *filename;
	char *name;
	char *locale;
   	char *group;
  	char *subgroup;
	char *image_filename[2];
	char *sound_filename;
	char *creation_date;
	char *revision_date;
	Eina_List *graphemics;
	Eina_List *neutrals;
	Eina_List *visuals;
	Eina_List *semantics;
} Item_Info;

//Define Patient Infos.
typedef struct
{
	char *filename;
	char *firstname;
	char *lastname;
	char *birthday;
	char *creation_date;
	char *revision_date;
} Patient_Info;

//Define Module Infos.
typedef struct
{
	Evas_Object *settings;
	char *id;
	char *name;
	char *description;
	char *icon;
	void (*_module_patient_set)(Patient_Info *patient);
} Module_Info;

//Define Statistic Infos.
typedef struct
{
   int module_id;
   int session_date;
   int nb_played;
   int score;
   int realization_time;
} Stat_Info;

//Item file handling functions.
extern Item_Info *item_read(const char *f);
extern const char *item_write(Item_Info *item);
extern int item_remove(Item_Info *item);
extern int item_update(Item_Info *item);
extern int item_import(Item_Info *item);
extern int item_export(Item_Info *item, const char *dst);
//extern int convert_sqlitedb_to_eet(App_Info *app, sqlite3 *db);   TODO:This functions should be used in an external tools and shipped with windows setup?

//Items list handling functions.
extern Eina_List *items_list_get();
extern Eina_List * items_list_free(Eina_List *items);
extern Eina_List *items_list_randomize_from_group(Eina_List *items, const char *group);
extern Eina_List *items_list_randomize(const Eina_List *items);
extern Eina_List *items_list_randomize_except(Eina_List *items, const char *name);
extern Eina_List *items_list_randomize_except_from_group(Eina_List *items, const char *group, const char *name);
extern Eina_List *items_list_randomize_with_group_and_subgroup(Eina_List *items, const char *group, const char *subgroup);
extern void items_list_print(const Eina_List *items);
extern void items_list_debug_test();

//Item handling functions.
extern Item_Info *item_new();
extern Item_Info *item_free(Item_Info *item);
extern void item_print(Item_Info *item);
extern const char *item_name_get(Item_Info *item);
extern const char *item_locale_get(Item_Info *item);
extern const char *item_group_get(Item_Info *item);
extern const char *item_sound_filename_get(Item_Info *item);
extern const char *item_subgroup_get(Item_Info *item);
extern void item_name_set(Item_Info *item, const char *name);
extern void item_locale_set(Item_Info *item, const char *locale);
extern void item_image_filename_set(Item_Info *item, const char *filename, const int level);
extern void item_sound_filename_set(Item_Info *item, const char *filename);
extern void item_group_set(Item_Info *item, const char *group);
extern void item_subgroup_set(Item_Info *item, const char *subgroup);
extern void item_distractor_add(Item_Info *item, const char *name, Distractor_Type type);
extern void item_distractor_clear(Item_Info *item, Distractor_Type type);
extern void item_distractors_print(const Item_Info *item);

//Group handling functions.
extern Group_Info *group_new();
extern Group_Info *group_free(Group_Info *group);
extern void group_name_set(Group_Info *group, const char *name);

//Groups list handling functions.
extern Eina_List *groups_list_get();
extern Eina_List *groups_list_free(Eina_List *groups);

//Modules data functions.
extern Module_Info *module_free(Module_Info *module);
extern Eina_List *modules_list_get(Evas_Object *obj);
extern Eina_List *modules_list_free(Eina_List *modules);

//path.c:FriLogos paths functions.
extern const char *frilogos_user_data_path_get (void);
extern const char *frilogos_system_data_path_get (void);
extern const char *frilogos_locale_path_get (void);
extern const char *frilogos_edje_theme_file_get (void);

//Patients file handling functions.
extern const char *patient_write(Patient_Info *patient);
extern int patient_remove(Patient_Info *patient);
extern Patient_Info *patient_read(const char *f);
extern int patient_update(Patient_Info *patient);

//Patients lists handling functions.
extern Eina_List *patients_list_get();
extern Eina_List *patients_list_free(Eina_List *patients);
extern void patients_list_debug_test();

//Patients handling functions.
extern Patient_Info *patient_new();
extern Patient_Info *patient_free(Patient_Info *patient);
extern void patient_print(Patient_Info *patient);
extern void patient_firstname_set(Patient_Info *patient, const char *firstname);
extern void patient_lastname_set(Patient_Info *patient, const char *lastname);
extern void patient_birthday_set(Patient_Info *patient, const char *birthday);
extern const char *patient_firstname_get(Patient_Info *patient);
extern const char *patient_lastname_get(Patient_Info *patient);
extern const char *patient_birthday_get(Patient_Info *patient);

//random.c: Randomize functions.
extern void RandomizeList(int *ptr, int n_elements);
extern int Prandom(int max);
extern void SeedRandomizer(void);

//settings.c: Randomize functions.
extern int setting_write(const char *setting, const int value);
extern int setting_int_read(const char *setting);
extern int file_setting_write(const char *file, const char *setting,  const int value);
extern int file_setting_int_read(const char *file, const char *setting);
extern Eina_Bool setting_bool_read(const char *setting);
extern const char *frilogos_settings_file_get(void);


//sound.c: Sound functions.
extern int PlaySoundFile(const char *SoundFile);

//Statistics handling functions.

//utils.c:Useful functions.
extern void evas_object_image_scale(Evas_Object *obj, int width, int height);
extern void window_clicked_close_cb(void *data, Evas_Object *obj __UNUSED__, void *event_info __UNUSED__);
extern const char *efreet_user_home_get(void);
extern char *trimwhitespace(char *str);
#endif /* __LIBFRILOGOS_H__ */
