/*
 * patients.c
 * This file is part of FriLogos
 *
 * Copyright (C) 2012 - Alexandre Dussart
 *
 * FriLogos is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * FriLogos is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FriLogos. If not, see <http://www.gnu.org/licenses/>.
 */

#include <Eet.h>
#include <Ecore_Evas.h>
#include <Ecore_File.h>

#include "libfrilogos.h"


//
//Read patient informations file.
//
Patient_Info *patient_read(const char *f)
{
	Patient_Info *patient;
  	Eet_File *ef;
  	char *ret;
	int   size;

	ef = eet_open(f, EET_FILE_MODE_READ);
	if(ef == NULL)
	{
		ERR(_("Can't open %s."), f);
		eet_close(ef);
		return NULL;
	}

	//INF(_("Reading %s."), f);
    patient = malloc(sizeof(*patient));

	//Read text informations about patient.
	patient->filename = strdup(f);

    ret = eet_read(ef, "FirstName", &size);
	patient_firstname_set(patient, ret);
    free(ret);

    ret = eet_read(ef, "LastName", &size);
	patient_lastname_set(patient, ret);
    free(ret);

    ret = eet_read(ef, "BirthDay", &size);
	patient_birthday_set(patient, ret);
    free(ret);

    ret = eet_read(ef, "Creation Date", &size);
	patient->creation_date = strdup(ret);
    free(ret);

	ret = eet_read(ef, "Revision Date", &size);
	if(ret)
		patient->revision_date = strdup(ret);
	else
		patient->revision_date = NULL;
    free(ret);

	//INF(_("%d fields read in %s file."), eet_num_entries(ef), patient->filename);

	eet_close(ef);

	return patient;
}


//
//Read all patients informations files.
//
extern Eina_List *patients_list_get()
{
	char s[PATH_MAX];
	const Eina_File_Direct_Info *f_info;
	Eina_Iterator *it;
	Eina_List *patients = NULL;
	Patient_Info *patient = NULL;

	snprintf(s, sizeof(s), "%spatients"DIR_SEPARATOR_S, frilogos_user_data_path_get());
	it = eina_file_stat_ls(s);

   if(it)
   {
	   EINA_ITERATOR_FOREACH(it, f_info)
	   {
            //INF("Found %s new file.", ecore_file_file_get(f_info->path));
			if(eina_str_has_extension(f_info->path, ".eet") == EINA_TRUE)
			{
				patient = patient_read(f_info->path);

				if(patient)
				{
					patients = eina_list_append(patients, patient);

					if (eina_error_get())
					{
						CRITICAL(_("Can't allocate list node."));
						exit(-1);
					}
				}
			}
		}

	eina_iterator_free(it);
	}

	return patients;
}


//
//Delete patient informations file.
//
int patient_remove(Patient_Info *patient)
{
    //INF("Deleting:%s", p->filename);
	ecore_file_remove(patient->filename);
	//free(p);
	return 0;
}


//
//Write patient informations file.
//
const char *patient_write(Patient_Info *patient)
{
	int i = 0;
	char s[PATH_MAX];
  	Eet_File *ef;
	time_t timestamp;
	struct tm *t;


	snprintf(s, sizeof(s), "%s"DIR_SEPARATOR_S"patients"DIR_SEPARATOR_S"%s%s-%03d.eet", frilogos_user_data_path_get(), patient->firstname, patient->lastname, i);

    INF(s);

	while(ecore_file_exists(s) == EINA_TRUE)
	{
		//strftime(buf, sizeof(buf),"%H%M%S", t);
	snprintf(s, sizeof(s), "%s"DIR_SEPARATOR_S"patients"DIR_SEPARATOR_S"%s%s-%03d.eet", frilogos_user_data_path_get(), patient->firstname, patient->lastname, i);
		i++;
	}

	patient->filename = strdup(s);
	ef = eet_open(patient->filename, EET_FILE_MODE_WRITE);

	if(ef == NULL)
	{
		ERR(_("Can't open %s."), patient->filename);
		eet_close(ef);
		return NULL;
	}

	eet_write(ef, "FirstName", patient_firstname_get(patient), strlen(patient_firstname_get(patient)) + 1, 1);
	eet_write(ef, "LastName", patient_lastname_get(patient), strlen(patient_lastname_get(patient)) + 1, 1);
	eet_write(ef, "BirthDay", patient_birthday_get(patient), strlen(patient_birthday_get(patient)) + 1, 1);

    //Add item creation informations.
	timestamp = time(NULL);
	t = localtime(&timestamp);

	snprintf(s, sizeof(s), "%02d-%02d-%d",
				(int)t->tm_mday,
  				(int)t->tm_mon,
  				1900+(int)t->tm_year);

	patient->creation_date = strdup(s);
	eet_write(ef, "Creation Date", s, strlen(s) + 1, 1);

	patient->revision_date = NULL;

	eet_sync(ef);
	eet_close(ef);

	return strdup(patient->filename);
}


//
//
//Update patient informations file.
//
int patient_update(Patient_Info *patient)
{
	char s[PATH_MAX];
  	Eet_File *ef;
	time_t timestamp;
	struct tm *t;

	ef = eet_open(patient->filename, EET_FILE_MODE_READ_WRITE);

	if(!ef)
	{
		ERR(_("Can't open %s."), patient->filename);
		eet_close(ef);
		return -1;
	}

	eet_write(ef, "FirstName", patient_firstname_get(patient), strlen(patient_firstname_get(patient)) + 1, 1);
	eet_write(ef, "LastName", patient_lastname_get(patient), strlen(patient_lastname_get(patient)) + 1, 1);
	eet_write(ef, "BirthDay", patient_birthday_get(patient), strlen(patient_birthday_get(patient)) + 1, 1);

	//Update patient revision date informations.
	timestamp = time(NULL);
	t = localtime(&timestamp);

	snprintf(s, sizeof(s), "%02d-%02d-%d",
				(int)t->tm_mday,
  				(int)t->tm_mon,
  				1900+(int)t->tm_year);
	eet_write(ef, "Revision Date", s, strlen(s) + 1, 1);
	patient->revision_date = strdup(s);

	eet_close(ef);

	return 0;
}



//Free patient allocated struct.
//
extern Patient_Info *patient_free(Patient_Info *patient)
{
	if(patient)
	{
		//INF("\tFreeing %s", patient->firstname);

		if(patient->filename)
		free(patient->filename);

		if(patient->lastname)
		free(patient->lastname);

		if(patient->firstname)
		free(patient->firstname);

		if(patient->birthday)
		free(patient->birthday);

		if(patient->creation_date)
		free(patient->creation_date);

		if(patient->revision_date)
		free(patient->revision_date);

		free(patient);

		return NULL;
	}
    return NULL;
}


//
//Free patients list.
//
Eina_List *patients_list_free(Eina_List *patients)
{
	if(patients)
	{
		Patient_Info *p;

        //Point to first node of list.
        for(patients = eina_list_last(patients); patients; patients = eina_list_prev(patients));

        EINA_LIST_FREE(patients, p)
			p = patient_free(p);

        eina_list_free(patients);

	   INF(_("%d patients registered."), eina_list_count(patients));
	    return NULL;
    }

    return NULL;
}

//
//Allocate&Initialize new patient struct.
//
Patient_Info *patient_new()
{
	Patient_Info *patient;

    patient = malloc(sizeof(*patient));
	patient->filename=NULL;
	patient_firstname_set(patient, NULL);
    patient_lastname_set(patient, NULL);
    patient_birthday_set(patient, NULL);
	patient->creation_date=NULL;
	patient->revision_date=NULL;

	return patient;
}


//
//Accessors functions.
//
const char *patient_firstname_get(Patient_Info *patient)
{
	if(patient->firstname)
        return strdup(patient->firstname);
	else
		return strdup(_("undefined"));
}


const char *patient_lastname_get(Patient_Info *patient)
{
	if(patient->lastname)
        return strdup(patient->lastname);
	else
		return strdup(_("undefined"));
}


const char *patient_birthday_get(Patient_Info *patient)
{
	if(patient->birthday)
		return strdup(patient->birthday);
	else
	{
    	char s[24];
    	time_t timestamp;
	    struct tm *t;

	    timestamp = time(NULL);
	    t = localtime(&timestamp);

    	snprintf(s, sizeof(s), "%02d-%02d-%d",
				(int)t->tm_mday,
  				(int)t->tm_mon,
  				1900+(int)t->tm_year);

    	return strdup(s);
    }
}


void patient_firstname_set(Patient_Info *patient, const char *firstname)
{

	if(firstname)
		patient->firstname = strdup(firstname);
	else
		patient->firstname = strdup(_("undefined"));
}


void patient_lastname_set(Patient_Info *patient, const char *lastname)
{
	if(lastname)
		patient->lastname = strdup(lastname);
	else
		patient->lastname = strdup(_("undefined"));
}


void patient_birthday_set(Patient_Info *patient, const char *birthday)
{
	if(birthday)
		patient->birthday = strdup(birthday);
	else
	{
    	char s[24];
    	time_t timestamp;
	    struct tm *t;

	    timestamp = time(NULL);
	    t = localtime(&timestamp);

    	snprintf(s, sizeof(s), "%02d-%02d-%d",
				(int)t->tm_mday,
  				(int)t->tm_mon,
  				1900+(int)t->tm_year);

    	patient->birthday = strdup(s);
    }
}


//
//Print patient informations.
//
void patient_print(Patient_Info *patient)
{
	INF(_("\nFilename:%s\n"
			"\t|->Patient: %s %s\n"
			"\t|->BirthDay: %s\n"
			"\t|->Creation Date:%s\n"
			"\t|->Revision Date:%s\n"),
			ecore_file_file_get(patient->filename),
			patient_firstname_get(patient),
			patient_lastname_get(patient),
			patient_birthday_get(patient),
			patient->creation_date,
			patient->revision_date);
}


//
//Debug patients handling funcs.
//
void patients_list_debug_test()
{
	int x;
	char s[12];
	Patient_Info *p;
	Eina_List *patients;
	Eina_List *l;


	INF("Now i'll create 25 patients with same name and different birthday...");

	for(x=0;x!=25;x++)
	{
		p = patient_new();
		patient_firstname_set(p, "John");
		patient_lastname_set(p, "Doe");


		snprintf(s, sizeof(s), "%02d-%02d-%d",
					(int)Prandom(28),
	  				(int)Prandom(12),
  					(int)1900+Prandom(110));

		patient_birthday_set(p, s);
		patient_write(p);
		patient_free(p);
	}

	INF("Now i'll list all patients file from frilogos data directory...");
	patients = patients_list_get();
	EINA_LIST_FOREACH(patients, l, p)
		patient_print(p);

	INF("Now i'll remove all previously generated patients files from frilogos data directory...");
	EINA_LIST_FOREACH(patients, l, p)
		patient_remove(p);

	patients = patients_list_free(patients);


	exit(1);
}
