/*
 * module.c
 * This file is part of Frilogos
 *
 * Copyright (C) 2012 - Alexandre Dussart
 *
 * Frilogos is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Frilogos is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Frilogos. If not, see <http://www.gnu.org/licenses/>.
 */


#include <Edje.h>


#include "module.h"
#include "frilogos.h"
#include "libfrilogos.h"

typedef enum 
{
       START_STATUS = 0,
       PAUSE_STATUS = 1,
       STOP_STATUS = 2
} Module_Status;


EAPI int version_maj_symbol = 0;
EAPI int version_min_symbol = 9;
EAPI int version_mic_symbol = 8;


// Internal's functions.
static void _navbar_resize(Module_Screen *screen);
static void _itemlabel_resize(Module_Screen *screen);
static void _distractors_resize(Module_Screen *screen);
static void _itempicture_resize(Module_Screen *screen);
static void _distractors_create(Module_Screen *screen);
static void _navbar_create(Module_Screen *screen);
static void _screen_update(Module_Screen *screen);
static void _navbar_hide(Module_Screen *screen);
static void _navbar_show(Module_Screen *screen);
static void _navbar_infos_text_set(Module_Screen *screen, const char *txt);
static void _navbar_infos_icon_set(Module_Screen *screen, Module_Status status);
static void _distractors_hide(Module_Screen *screen);
static void _distractors_show(Module_Screen *screen);

static Evas_Object *_get_evas_object_from_edje(Evas_Object *edje, const char *object_name);


// Evas callbacks.
static void window_close_cb(void *data, Evas *e, Evas_Object *obj, void *event_info);
static Eina_Bool timer1_cb(void *data);
static Eina_Bool timer2_cb(void *data);
static Eina_Bool timer3_cb(void *data);
static void keydown_cb(void *data, Evas *e, Evas_Object *obj, void *event_info);
static void wrong_cb(void *data, Evas *e, Evas_Object *obj, void *event_info);
static void right_cb(void *data, Evas *e, Evas_Object *obj, void *event_info);
static void ecore_evas_resize_cb(Ecore_Evas *ee);


EAPI void _print_module_infos(void)
{
	INF(_("Diaporama module"));
	INF(_("\tVersion:%d.%d.%d"),
					version_maj_symbol,
					version_min_symbol,
					version_mic_symbol);
	INF(_("\tBuild on:%s"), __DATE__);

}

EAPI char *_module_name_get(void)
{
	return  strdup(_("Diaporama"));
}


EAPI char *_module_desc_get(void)
{
	return strdup(_("Items appears and patient must memorize it."));
}


EAPI char *_module_id_get(void)
{
	return strdup(_("DIAPORAMA"));
}


EAPI char *_module_theme_get(void)
{
	char p[FILENAME_MAX];

	snprintf(p, sizeof(p), "%s%sdiaporama.edj", THEMEDIR, DIR_SEPARATOR_S);

	return strdup(p);
}


//
//Show item after timer 2.
//Used when timers are enabled.
static Eina_Bool timer3_cb(void *data)
{
	Module_Screen *screen = data;

    _distractors_show(screen);
    _navbar_show(screen);

	//Delete current timer to avoid auto-repeat.
	return ECORE_CALLBACK_CANCEL;
}



//
//Timer used to hide anything from diaporama screen.
//Used when timers are enabled.
static Eina_Bool timer2_cb(void *data)
{
	Module_Screen *screen = data;

	//Don't show anything according to hiding timer wanted.
    evas_object_image_file_set(screen->item_img, NULL, NULL);
    evas_image_cache_flush(screen->evas);

    //
  	screen->timer2 = ecore_timer_add(screen->settings->hiding_timer_wanted, timer3_cb, screen);

	//Delete current timer to avoid auto-repeat.
	return ECORE_CALLBACK_CANCEL;
}


static Eina_Bool timer1_cb(void *data)
{
	Module_Screen *screen = data;

	_screen_update(screen);

	//Delete current timer to avoid auto-repeat.
	return ECORE_CALLBACK_CANCEL;
}




static void right_cb(void *data, Evas *e __UNUSED__, Evas_Object *obj, void *event_info __UNUSED__)
{
	int *answerpos;
	int x, y, w, h;
	char s[128];
	Module_Screen *screen = data;

	screen->score++;

	if(screen->settings->navbar_wanted == EINA_TRUE)
	{
            evas_object_image_fill_get(obj, NULL, NULL, &w, &h);
            evas_object_image_scale(obj, w*0.90, h*0.90);
            evas_object_show(obj);
		    snprintf(s, sizeof(s), _("Score:%d/%d"), screen->score, screen->nbserial_played);
            _navbar_infos_text_set(screen, s);
            _navbar_infos_icon_set(screen, PAUSE_STATUS);

  		    //Disable navbar events to prevent other user's actions.
        	evas_object_event_callback_del_full(screen->right_bt , EVAS_CALLBACK_MOUSE_DOWN, right_cb, screen);
       		evas_object_event_callback_del_full(screen->wrong_bt , EVAS_CALLBACK_MOUSE_DOWN, wrong_cb, screen);
        	evas_object_event_callback_del_full(screen->shutdown_bt, EVAS_CALLBACK_MOUSE_DOWN, window_close_cb, screen);
	}


	//If distractors are enabled, display correct answer.
	if(screen->settings->distractors_wanted == EINA_TRUE)
	{
		answerpos = evas_object_data_get(screen->bg, "answerpos");
  		evas_object_color_set(screen->correction, 100, 255, 0, 150);
		evas_object_geometry_get(screen->distractors_frame[*answerpos], &x, &y, &w, &h);
		evas_object_move(screen->correction, x, y);
		evas_object_image_fill_set(screen->correction, x, y, w, h);
		evas_object_resize(screen->correction, w, h);
  		evas_object_show(screen->correction);

  		//Disable distractors events to prevent other user action and cleaning old callbacks.
  		for(x = 0 ; x < 4; x++)
  		{
			evas_object_event_callback_del_full(screen->distractors_text[x], EVAS_CALLBACK_MOUSE_DOWN, right_cb, screen);
			evas_object_event_callback_del_full(screen->distractors_frame[x], EVAS_CALLBACK_MOUSE_DOWN, right_cb, screen);
			evas_object_event_callback_del_full(screen->distractors_text[x], EVAS_CALLBACK_MOUSE_DOWN, wrong_cb, screen);
			evas_object_event_callback_del_full(screen->distractors_frame[x], EVAS_CALLBACK_MOUSE_DOWN, wrong_cb, screen);
  		}
	}
  	screen->timer1 = ecore_timer_add(0.1, timer1_cb, screen);
}


static void wrong_cb(void *data, Evas *e __UNUSED__, Evas_Object *obj __UNUSED__, void *event_info __UNUSED__)
{
	int *answerpos;
	int x, y, w, h;
	char s[128];
	Module_Screen *screen = data;


	if(screen->settings->navbar_wanted == EINA_TRUE)
	{
        evas_object_image_fill_get(obj, NULL, NULL, &w, &h);
        evas_object_image_scale(obj, w*0.90, h*0.90);
        evas_object_show(obj);
            	
		snprintf(s, sizeof(s), _("Score:%d/%d"), screen->score, screen->nbserial_played);
        _navbar_infos_text_set(screen, s);
        _navbar_infos_icon_set(screen, PAUSE_STATUS);

  		//Disable navbar events to prevent other user's actions.
		evas_object_event_callback_del_full(screen->right_bt, EVAS_CALLBACK_MOUSE_DOWN, right_cb, screen);
		evas_object_event_callback_del_full(screen->wrong_bt, EVAS_CALLBACK_MOUSE_DOWN, wrong_cb, screen);
		evas_object_event_callback_del_full(screen->shutdown_bt, EVAS_CALLBACK_MOUSE_DOWN, window_close_cb, screen);
	}

	//If distractors are enabled, display correct answer in green.
	if(screen->settings->distractors_wanted == EINA_TRUE)
	{
		answerpos = evas_object_data_get(screen->bg, "answerpos");
  		evas_object_color_set(screen->correction, 255, 0, 55, 150);
		evas_object_geometry_get(screen->distractors_frame[*answerpos], &x, &y, &w, &h);
		evas_object_image_fill_set(screen->correction, x, y, w, h);
		evas_object_resize(screen->correction, w, h);
		evas_object_move(screen->correction, x, y);
  		evas_object_show(screen->correction);
        free(evas_object_data_del(screen->bg, "answerpos"));


  		//Disable distractors events to prevent other user action and cleaning old callbacks.
  		for(x = 0 ; x < 4; x++)
  		{
			evas_object_event_callback_del_full(screen->distractors_text[x], EVAS_CALLBACK_MOUSE_DOWN, right_cb, screen);
			evas_object_event_callback_del_full(screen->distractors_frame[x], EVAS_CALLBACK_MOUSE_DOWN, right_cb, screen);
			evas_object_event_callback_del_full(screen->distractors_text[x], EVAS_CALLBACK_MOUSE_DOWN, wrong_cb, screen);
			evas_object_event_callback_del_full(screen->distractors_frame[x], EVAS_CALLBACK_MOUSE_DOWN, wrong_cb, screen);
  		}
	}

	if(screen->settings->timers_wanted == EINA_TRUE)
	{
	    Item_Info *item;
	    item = evas_object_data_get(screen->bg, "item");
	    snprintf(s, sizeof(s), "images/%d", screen->settings->imglvl_wanted);
	    evas_object_image_file_set(screen->item_img, item->filename, s);
	    _itempicture_resize(screen);
	}

	screen->timer1 = ecore_timer_add(0.1, timer1_cb, screen);
}




static void window_close_cb(void *data __UNUSED__, Evas *e __UNUSED__, Evas_Object *obj __UNUSED__, void *event_info __UNUSED__)
{
    int w, h;
    
    evas_object_image_fill_get(obj, NULL, NULL, &w, &h);
    evas_object_image_scale(obj, w*0.90, h*0.90);
    evas_object_show(obj);
	ecore_main_loop_quit();
}




static int _sort_cb()
{
    return Prandom(2) - 1;
}



static void _distractors_set(Module_Screen *screen, Item_Info *item)
{
        int x, s, tries;
        char *dis[3];
        int l1, l2, l3, l4;
        Eina_List *distractors;

        for(x = 0; x < 3;x++)
            dis[x] = NULL;

        INF(_("Setting distractor for item %s."), item->name);

        int num_semantics = eina_list_count(item->semantics);
        int num_graphemics = eina_list_count(item->graphemics);
        int num_visuals = eina_list_count(item->visuals);
        int num_neutrals = eina_list_count(item->neutrals);
                
        item->semantics = eina_list_sort(item->semantics , eina_list_count(item->semantics ), EINA_COMPARE_CB(_sort_cb));
        item->visuals = eina_list_sort(item->visuals , eina_list_count(item->visuals ), EINA_COMPARE_CB(_sort_cb));	
        item->graphemics = eina_list_sort(item->graphemics , eina_list_count(item->graphemics ), EINA_COMPARE_CB(_sort_cb));
        item->neutrals = eina_list_sort(item->neutrals , eina_list_count(item->neutrals), EINA_COMPARE_CB(_sort_cb));        
              
        
        l1 = l2 = l3 = l4 = 0;   
                        	        
        tries=x=0;
        while(x < 3 && tries != 16)
        {
            s = Prandom(3)-1;
        
            switch(s)
            {
                case 0:
               	    if(screen->settings->semantics_wanted == EINA_TRUE && num_semantics > 0)
               	    {
                        if(l1<num_semantics)
                        {   
               	            dis[x] = eina_list_nth(item->semantics, l1);
               	            l1++;
               	            x++;
               	        }
                    }
                    break;
                case 1:
               	if(screen->settings->graphemics_wanted == EINA_TRUE  && num_graphemics > 0 )
               	    {
                        if(l2<num_graphemics)
                        { 
                            dis[x] = eina_list_nth(item->graphemics, l2);
                            l2++;
                            x++;
                        }
                    }
                    break;
                case 2:
                if(screen->settings->visuals_wanted == EINA_TRUE && num_visuals > 0)
                    {
                         if(l3<num_visuals)
                        { 
                            dis[x] = eina_list_nth(item->visuals, l3);
                            l3++;
               	            x++;
               	        }
                        break;
                    }
            }        
            tries++;                
        }

    //If no distractors set with these item, try to found in neutral distractor type or at least solution generate them.
    distractors = items_list_randomize_except_from_group(screen->_items, item->group, item->name);  
    if(!distractors)
        distractors = items_list_randomize_except(screen->_items,item->name);  
    if(!distractors)
    {
        ERR(_("Can't generate distractors, not enough items registered!"));
        ecore_main_loop_quit();
    }
    for(x = 0; x < 3;x++)
    {   
            if(!dis[x])
            {
                if(num_neutrals > 0)
                {              
                    if(l4<num_neutrals)
                    { 	        
                        dis[x] = eina_list_nth(item->neutrals, l4);
                        l4++;
                    }
                }

             if(!dis[x])
             {
                    Item_Info *distractor;
                    distractor = eina_list_data_get(distractors);
                    dis[x] = distractor->name;
    				distractors = eina_list_next(distractors); 
            }
        }                
     }
    
    distractors = items_list_free(distractors);
    
	int *answerpos;
	answerpos = evas_object_data_get(screen->bg, "answerpos");
    
    int y = 0;
    for(x = 0; x < 4;x++)
	{
    		if(x != *answerpos)
		    {           	        	
            	evas_object_text_text_set(screen->distractors_text[x] , elm_entry_markup_to_utf8((dis[y])));
				evas_object_event_callback_add(screen->distractors_text[x] , EVAS_CALLBACK_MOUSE_DOWN, wrong_cb, screen);
				evas_object_event_callback_add(screen->distractors_frame[x] , EVAS_CALLBACK_MOUSE_DOWN, wrong_cb, screen);
                y++;
    		}
	}
}


static void _screen_update(Module_Screen *screen)
{
	int *answerpos;
	char s[128];
	Item_Info *item;


	//Stop Game when reached parameted serial.
	if(screen->nbserial_played == screen->settings->nbserial_wanted)
	{
		ecore_main_loop_quit();
		return;
	}

	screen->nbserial_played ++;
    
	item = eina_list_data_get(screen->items);
	if(!item)
	{
		ecore_main_loop_quit();
		return;
	}
	evas_object_data_set(screen->bg, "item", item);	

	screen->items = eina_list_next(screen->items);

    //Cleanup evas image's memory.
    evas_object_image_file_set(screen->item_img, NULL, NULL);
    evas_image_cache_flush(screen->evas);
    snprintf(s, sizeof(s), "images/%d", screen->settings->imglvl_wanted);
    evas_object_image_file_set(screen->item_img, item->filename, s);

    Evas_Load_Error err = evas_object_image_load_error_get(screen->item_img);
    if (err != EVAS_LOAD_ERROR_NONE)
    {
      ERR(_("Can't load Edje image from %s!"), item->filename);
         _screen_update(screen);
    }

	_itempicture_resize(screen);

	if(screen->settings->itemlabel_wanted  == EINA_TRUE)
	{
		evas_object_text_text_set(screen->name_text , elm_entry_markup_to_utf8(item->name));
		_itemlabel_resize(screen);
	}


	if(screen->settings->distractors_wanted == EINA_TRUE)
	{
        answerpos = malloc(sizeof(*answerpos));
	    *answerpos = Prandom(3);
		evas_object_data_set(screen->bg, "answerpos", answerpos);
		evas_object_text_text_set(screen->distractors_text[*answerpos] , elm_entry_markup_to_utf8(item->name));
       //Set new callbacks according to correct answer.
		evas_object_event_callback_add(screen->distractors_text[*answerpos] , EVAS_CALLBACK_MOUSE_DOWN, right_cb, screen);
		evas_object_event_callback_add(screen->distractors_frame[*answerpos] , EVAS_CALLBACK_MOUSE_DOWN, right_cb, screen);
        _distractors_set(screen, item);

 		_distractors_resize(screen);
		evas_object_hide(screen->correction);
	}


	if(screen->settings->navbar_wanted == EINA_TRUE)
	{
	    _navbar_resize(screen);
	   	snprintf(s, sizeof(s), _("Sequence %d"), screen->nbserial_played);
        _navbar_infos_text_set(screen, s);
        _navbar_infos_icon_set(screen, START_STATUS);

       	evas_object_event_callback_add(screen->right_bt , EVAS_CALLBACK_MOUSE_DOWN, right_cb, screen);
      	evas_object_event_callback_add(screen->wrong_bt , EVAS_CALLBACK_MOUSE_DOWN, wrong_cb, screen);
       	evas_object_event_callback_add(screen->shutdown_bt, EVAS_CALLBACK_MOUSE_DOWN, window_close_cb, screen);
	}

	if(screen->settings->timers_wanted == EINA_TRUE)
	{
        _navbar_hide(screen);

        _distractors_hide(screen);
	  	screen->timer2 = ecore_timer_add(screen->settings->exposition_timer_wanted, timer2_cb, screen);
	}
	
	//For testing purpose only
    //Generate a random answer.
    //int a=0;
    //a = Prandom(2);
    //    switch(a)
    //    {
    //        case 1:
    //            right_cb(screen, NULL, NULL, NULL);   
    //            break;         
    //        case 2:
    //            wrong_cb(screen, NULL, NULL, NULL);
    //            break;         
    //    }
}



static void keydown_cb(void *data, Evas *e __UNUSED__, Evas_Object *obj __UNUSED__, void *event_info)
{
	Evas_Event_Key_Down *event;
	Module_Screen *screen = data;

	event = (Evas_Event_Key_Down *)event_info;

	if ( !strcmp(event->keyname, "Escape"))
	{
		if(ecore_evas_fullscreen_get(screen->ee))
			ecore_evas_fullscreen_set(screen->ee, EINA_FALSE);
		else
		ecore_evas_fullscreen_set(screen->ee, EINA_TRUE);
	}
}



static void _itempicture_resize(Module_Screen *screen)
{
	int itemw, itemh;
	int diapoh, diapow;

    evas_object_image_smooth_scale_set(screen->item_img, EINA_TRUE);
	evas_object_image_fill_get(screen->diapo_img, NULL, NULL, &diapow, &diapoh);
	evas_object_image_scale(screen->item_img, diapow*0.70, diapoh *0.70);
	evas_object_image_fill_get(screen->item_img, NULL, NULL, &itemw, &itemh);
	evas_object_move(screen->item_img, (screen->geometry.w - itemw) / 2,  (diapoh - itemh) / 2);
    evas_object_show (screen->item_img);
}


static void _itemlabel_resize(Module_Screen *screen)
{
	int itemw, itemh, w, h, x, y;
	int diapoh;

	//First get diaporama height to help to move distractors frame below it.
	evas_object_image_fill_get(screen->diapo_img, NULL, NULL, NULL, &diapoh);

 	//Scale and move item name stuff.
    evas_object_image_scale(screen->name_frame, screen->geometry.w*0.3, screen->geometry.h*0.5);
    evas_object_image_fill_get(screen->name_frame, NULL, NULL, &w, &h);
    evas_object_move(screen->name_frame, (screen->geometry.w-w)/2, diapoh + h*0.2);
    evas_object_text_font_set(screen->name_text, "Vera-Bold", h*0.5);
	evas_object_geometry_get(screen->name_frame, &x, &y, &w, &h);
    evas_object_geometry_get(screen->name_text, NULL, NULL, &itemw, &itemh);
	evas_object_move(screen->name_text, x + ((w-itemw)/2), y+((h-itemh)/2));
}


static void _distractors_resize(Module_Screen *screen)
{
	int i;
	int itemw, itemh, w, h, x, y;
	int diapoh;

	//First get diaporama height to help to move distractors frame below it.
	evas_object_image_fill_get(screen->diapo_img, NULL, NULL, NULL, &diapoh);

 	//Scale and move distractors stuff.
	for(i = 0; i < 4; i++)
	{
	       	evas_object_image_scale(screen->distractors_frame[i], screen->geometry.w*0.3, screen->geometry.h*0.5);
	       	evas_object_image_fill_get(screen->distractors_frame[i], NULL, NULL, &w, &h);
		    evas_object_text_font_set(screen->distractors_text[i], "Vera-Bold", h*0.5);

	       	switch(i)
	       	{
	       		case 0: evas_object_move(screen->distractors_frame[i], (w+50)/2, diapoh + h*0.2);
	       			break;
	       		case 1: evas_object_move(screen->distractors_frame[i], (w+50)/2+w, diapoh + h*0.2);
				break;
	       		case 2: evas_object_move(screen->distractors_frame[i], (w+50)/2, diapoh + h  + 20 + h*0.2);
	       			break;
	       		case 3: evas_object_move(screen->distractors_frame[i], (w+50)/2+w, diapoh + 20 + h + h*0.2);
	       	}

	       	evas_object_geometry_get(screen->distractors_frame[i], &x, &y, &w, &h);
       		evas_object_geometry_get(screen->distractors_text[i], NULL, NULL, &itemw, &itemh);
	       	evas_object_move(screen->distractors_text[i], x + ((w-itemw)/2),  y+((h-itemh)/2));
	}
 }



static void _navbar_resize(Module_Screen *screen)
{
    int w, h;
    int itemw, itemh;

	evas_object_image_scale(screen->bar_img, screen->geometry.w*0.5, screen->geometry.h*0.2);
	evas_object_image_fill_get(screen->bar_img, NULL, NULL, &w, &h);
	evas_object_move(screen->bar_img, 0, screen->geometry.h - h);

	evas_object_text_font_set(screen->infos_text  , "Vera-Bold",  h*0.4);
	evas_object_geometry_get(screen->infos_text, NULL, NULL, &itemw, &itemh);
	evas_object_move(screen->infos_text, w * 0.17,  screen->geometry.h - ((h + itemh) / 2));

    evas_object_image_scale(screen->status_img[START_STATUS], w*0.5, h*0.5);
	evas_object_geometry_get(screen->status_img[START_STATUS], NULL, NULL, &itemw, &itemh);
	evas_object_move(screen->status_img[START_STATUS], w * 0.05, screen->geometry.h - ((h + itemh) / 2));

    evas_object_image_scale(screen->status_img[PAUSE_STATUS], w*0.5, h*0.5);
	evas_object_geometry_get(screen->status_img[PAUSE_STATUS], NULL, NULL, &itemw, &itemh);
	evas_object_move(screen->status_img[PAUSE_STATUS], w * 0.05, screen->geometry.h - ((h + itemh) / 2));

    evas_object_image_scale(screen->status_img[STOP_STATUS], w*0.5, h*0.5);
	evas_object_geometry_get(screen->status_img[STOP_STATUS], NULL, NULL, &itemw, &itemh);
	evas_object_move(screen->status_img[STOP_STATUS], w * 0.05, screen->geometry.h - ((h + itemh) / 2));

    evas_object_image_scale(screen->right_bt, w*0.5, h*0.5);
	evas_object_geometry_get(screen->right_bt, NULL, NULL, &itemw, &itemh);
	evas_object_move(screen->right_bt, w * 0.75, screen->geometry.h - ((h + itemh) / 2));

    evas_object_image_scale(screen->wrong_bt, w*0.5, h*0.5);
	evas_object_geometry_get(screen->wrong_bt, NULL, NULL, &itemw, &itemh);
	evas_object_move(screen->wrong_bt, w * 0.82, screen->geometry.h - ((h + itemh) / 2));

    evas_object_image_scale(screen->shutdown_bt, w*0.5, h*0.5);
	evas_object_geometry_get(screen->shutdown_bt, NULL, NULL, &itemw, &itemh);
	evas_object_move(screen->shutdown_bt, w * 0.90, screen->geometry.h - ((h + itemh) / 2));
}



static void _distractors_create(Module_Screen *screen)
{
	int i;


	screen->distractors_frame[0] = _get_evas_object_from_edje(screen->edje, "distractor0_frame");
	screen->distractors_frame[1] = _get_evas_object_from_edje(screen->edje, "distractor1_frame");
	screen->distractors_frame[2] = _get_evas_object_from_edje(screen->edje, "distractor2_frame");
	screen->distractors_frame[3] = _get_evas_object_from_edje(screen->edje, "distractor3_frame");

	for(i = 0; i < 4; i++)
	{
 	    evas_object_show(screen->distractors_frame[i]);

 		screen->distractors_text[i] = evas_object_text_add(screen->evas);
		evas_object_text_style_set(screen->distractors_text[i], EVAS_TEXT_STYLE_SOFT_OUTLINE);
		evas_object_text_text_set(screen->distractors_text[i], _("Sample"));
		//evas_object_text_outline_color_set(screen->distractors_text[i], 0, 0, 0, 255);
		evas_object_color_set(screen->distractors_text[i], 0, 0, 0, 255);
		evas_object_show(screen->distractors_text[i]);
	}

	screen->correction = evas_object_rectangle_add(screen->evas);
}

static void _navbar_create(Module_Screen *screen)
{
 	screen->bar_img = _get_evas_object_from_edje(screen->edje, "navbar");
	evas_object_show(screen->bar_img);

	screen->infos_text = evas_object_text_add(screen->evas);
	evas_object_text_text_set(screen->infos_text, _("Diaporama"));
	evas_object_color_set(screen->infos_text, 0, 0, 0, 255);
	evas_object_show(screen->infos_text);

 	screen->status_img[START_STATUS] = _get_evas_object_from_edje(screen->edje, "start");
 	screen->status_img[PAUSE_STATUS] = _get_evas_object_from_edje(screen->edje, "pause");
 	screen->status_img[STOP_STATUS] = _get_evas_object_from_edje(screen->edje, "stop");

	screen->right_bt = _get_evas_object_from_edje(screen->edje, "right");
	evas_object_show(screen->right_bt);

	screen->wrong_bt = _get_evas_object_from_edje(screen->edje, "wrong");
	evas_object_show(screen->wrong_bt);

	screen->shutdown_bt = _get_evas_object_from_edje(screen->edje, "shutdown");
	evas_object_show(screen->shutdown_bt);
}



static void ecore_evas_resize_cb(Ecore_Evas *ee)
{
    int diapow, diapoh;
    int w, h;

    Module_Screen *screen;

    screen = (Module_Screen *)evas_data_attach_get(ecore_evas_get(ee));

	ecore_evas_geometry_get(ee, NULL, NULL, &w, &h);
	screen->geometry.w = w;
	screen->geometry.h = h;

	evas_object_image_fill_set(screen->bg, 0, 0, screen->geometry.w, screen->geometry.h);
	evas_object_resize(screen->bg, screen->geometry.w, screen->geometry.h);

	//Scale and move diaporama.
	evas_object_image_scale(screen->diapo_img, screen->geometry.w*0.9, screen->geometry.h*0.75);
	evas_object_image_fill_get(screen->diapo_img, NULL, NULL, &diapow, &diapoh);
	evas_object_move(screen->diapo_img, (screen->geometry.w - diapow) / 2, 0);

	//Scale and move item picture.
    _itempicture_resize(screen);

	//Scale and move navigation bar.
	if(screen->settings->navbar_wanted == EINA_TRUE)
		_navbar_resize(screen);

	//Scale and move item distractor's frames.
	if(screen->settings->distractors_wanted == EINA_TRUE)
		_distractors_resize(screen);

	//Scale and move item label frame.
	if(screen->settings->itemlabel_wanted == EINA_TRUE)
		_itemlabel_resize(screen);
}


static void _distractors_show(Module_Screen *screen)
{
    //Don't try to  show if distractors option isn't set.
	if(screen->settings->distractors_wanted == EINA_TRUE)
	{
        int x;
	    for(x = 0 ; x < 4; x++)
  	    {
		    evas_object_show(screen->distractors_text[x]);
		    evas_object_show(screen->distractors_frame[x]);
  	    }
  	 }
}


static void _distractors_hide(Module_Screen *screen)
{
    //Don't try to  hide if distractors option isn't set.
	if(screen->settings->distractors_wanted == EINA_TRUE)
	{
        int x;
    	for(x = 0 ; x < 4; x++)
      	{
    		evas_object_hide(screen->distractors_text[x]);
    		evas_object_hide(screen->distractors_frame[x]);
      	}
    }
}


static void _navbar_show(Module_Screen *screen)
{
    //Don't try to  show if navbar option isn't set.
	if(screen->settings->navbar_wanted == EINA_TRUE)
	{
	    evas_object_show(screen->bar_img);
		evas_object_show(screen->infos_text);
		evas_object_show(screen->status_img[START_STATUS]);
		evas_object_show(screen->status_img[PAUSE_STATUS]);
		evas_object_show(screen->status_img[STOP_STATUS]);
		evas_object_show(screen->right_bt);
		evas_object_show(screen->wrong_bt);
		evas_object_show(screen->shutdown_bt);
	}
}


//
//Hide navbar.
//
static void _navbar_hide(Module_Screen *screen)
{
    //Don't try to hide if navbar option isn't set.
	if(screen->settings->navbar_wanted == EINA_TRUE)
	{
	    evas_object_hide(screen->bar_img);
		evas_object_hide(screen->infos_text);
		evas_object_hide(screen->status_img[START_STATUS]);
		evas_object_hide(screen->status_img[PAUSE_STATUS]);
		evas_object_hide(screen->status_img[STOP_STATUS]);
		evas_object_hide(screen->right_bt);
		evas_object_hide(screen->wrong_bt);
		evas_object_hide(screen->shutdown_bt);
    }
}

//
//Set text in navbar infos panel.
//
static void _navbar_infos_text_set(Module_Screen *screen, const char *txt)
{
    //Don't try to set text if navbar option isn't set.
	if(screen->settings->navbar_wanted == EINA_TRUE)
	{
    	evas_object_text_text_set(screen->infos_text , txt);
	}
}

//
//Set icon in navbar infos panel.
//
static void _navbar_infos_icon_set(Module_Screen *screen, Module_Status status)
{
    //Don't try to set icon if navbar option isn't set.
	if(screen->settings->navbar_wanted == EINA_TRUE)
	{
        evas_object_hide(screen->status_img[START_STATUS]);
        evas_object_hide(screen->status_img[PAUSE_STATUS]);
        evas_object_hide(screen->status_img[STOP_STATUS]);

        evas_object_show(screen->status_img[status]);
	}
}



void _module_print_settings(Module_Screen *screen)
{
    char *s;
    Eina_List *sel, *l;
    Elm_Object_Item *it;    
    
    Module_Settings *settings = screen->settings;

    printf("\n\nDiaporama Settings\n"
            "\tNavigation Bar:%s\n"
            "\tTimers:%s\n"
            "\tItem label:%s\n"
            "\tExposition timer set to:%f sec\n"
            "\tHiding timer set to:%f sec\n"
            "\tNb serial:%f\n",
            settings->navbar_wanted? "yes":"no",
            settings->timers_wanted? "yes":"no",
            settings->itemlabel_wanted? "yes":"no",
            settings->exposition_timer_wanted,
            settings->hiding_timer_wanted,
            settings->nbserial_wanted);

	if((screen->settings->semantics_wanted == EINA_TRUE) ||
	(screen->settings->graphemics_wanted == EINA_TRUE) ||
	(screen->settings->visuals_wanted == EINA_TRUE))
        screen->settings->distractors_wanted = EINA_TRUE;
    else
        screen->settings->distractors_wanted = EINA_FALSE;
	printf("\tDistractors:%s\n", screen->settings->distractors_wanted?"yes":"no");
	    
	if(screen->settings->distractors_wanted == EINA_TRUE)
    printf("\tGraphemics:%s\n"
            "\tVisuals :%s\n"
            "\tSemantics :%s\n",          
            settings->graphemics_wanted? "enabled":"disabled",
            settings->visuals_wanted? "enabled":"disabled",
            settings->semantics_wanted? "enabled":"disabled");


    sel =  elm_list_selected_items_get(settings->groups_list);

    if(!sel || eina_list_count(sel)==0)
    {
        INF(_("Playing with all groups."));
        screen->items = items_list_randomize(screen->_items);
    }
    else
    {
    EINA_LIST_FOREACH(sel, l, it)
    {
        s = (char *)elm_object_item_data_get(it);
        if(s)
        {
            if(strcmp(s, _("All groups")) == 0)
            {
                    INF(_("Playing with all groups."));            
		            screen->items = items_list_randomize(screen->_items);
		            break;
		    }
        
            char **tmp;
            Eina_List *l2;
            tmp = eina_str_split(s, "/", 0);           
            l2 = items_list_randomize_with_group_and_subgroup(screen->_items, tmp[0], tmp[1]);
            screen->items = eina_list_merge(screen->items, l2);
            l2 = items_list_free(l2);
            free(tmp[0]);
            free(tmp);
       }
    }
    }
    screen->items = eina_list_sort(screen->items , eina_list_count(screen->items), EINA_COMPARE_CB(_sort_cb));

	if(screen->settings->patient_selected)
		INF(_("Playing with:%s."), screen->settings->patient_selected->firstname);
	else
		INF(_("Playing with unknown patient."));    
}





void _module_screen_create_cb(void *data, Evas_Object *obj __UNUSED__, void *event_info __UNUSED__)
{
	char s[PATH_MAX];
	Module_Screen *screen = NULL;

    if(screen)
        return;

    screen = malloc(sizeof(*screen));

	INF("Diaporama game.");

	//Initialize statistics default values
	screen->nbserial_played = 0;
	screen->score = 0;
	screen->realization_time = 0;
	screen->_items = items_list_get();
	screen->items = NULL;	
	screen->settings = (Module_Settings *) data;

    _module_print_settings(screen);

    //Avoid multiple executions of these module.
    elm_object_disabled_set(screen->settings->exec_bt, EINA_TRUE);

	screen->geometry.w = 0;
	screen->geometry.h = 0;
	screen->geometry.x = 0;
	screen->geometry.y = 0;

        if ((screen->geometry.w <= 0) && (screen->geometry.h <= 0))
        {
			screen->geometry.w = DEFAULT_WIDTH;
        	screen->geometry.h = DEFAULT_HEIGHT;
        }

	#ifdef HAVE_EVIL
	screen->ee = ecore_evas_new ("software_gdi", 0, 0, screen->geometry.w, screen->geometry.h , NULL);
	#else
	screen->ee = ecore_evas_new ("software_x11", 0, 0, screen->geometry.w, screen->geometry.h , NULL);
	#endif


	if (!screen->ee)
	{
		CRITICAL(_("Can't construct Ecore_Evas object!"));
		return;
	}

	ecore_evas_fullscreen_set(screen->ee, EINA_TRUE);
	ecore_evas_callback_resize_set(screen->ee, ecore_evas_resize_cb);
	ecore_evas_title_set(screen->ee, _("Diaporama"));
	ecore_evas_show(screen->ee);

	screen->evas = ecore_evas_get(screen->ee);
	evas_data_attach_set(screen->evas, screen);

	screen->edje = edje_object_add(screen->evas);
	if (!screen->edje)
	{
		CRITICAL(_("Can't create Edje object!"));
		return;
	}

	if (!edje_object_file_set(screen->edje, _module_theme_get(), "ui"))
	{
		ERR(_("Can't load Edje object from %s!"), s);
		evas_object_del(screen->edje);
	}
	evas_object_move(screen->edje, 0, 0);
	evas_object_resize(screen->edje, screen->geometry.w, screen->geometry.h);
	evas_object_show(screen->edje);


 	screen->bg = _get_evas_object_from_edje(screen->edje, "background");
	evas_object_focus_set(screen->bg, EINA_TRUE);
	evas_object_event_callback_add(screen->bg, EVAS_CALLBACK_KEY_DOWN, keydown_cb, screen);
	evas_object_resize(screen->bg, screen->geometry.w, screen->geometry.h);
    evas_object_show(screen->bg);

 	screen->diapo_img = _get_evas_object_from_edje(screen->edje, "diaporama");
    evas_object_show(screen->diapo_img);

 	screen->item_img = _get_evas_object_from_edje(screen->edje, "unknown");
    evas_object_show(screen->item_img);


	if(screen->settings->navbar_wanted == EINA_TRUE)
	_navbar_create(screen);

		if(screen->settings->itemlabel_wanted == EINA_TRUE)
		{
		 	screen->name_frame = _get_evas_object_from_edje(screen->edje, "itemlabel_frame");
			evas_object_show(screen->name_frame);
        	screen->name_text = evas_object_text_add(screen->evas);
			evas_object_text_font_set(screen->name_text, "Vera-Bold", 20);
			evas_object_text_text_set(screen->name_text, "Test item name");
			evas_object_color_set(screen->name_text, 0, 0, 0, 255);
			evas_object_show(screen->name_text);
		}

	if((screen->settings->distractors_wanted == EINA_TRUE))
		_distractors_create(screen);

    
	_screen_update(screen);

	ecore_main_loop_begin();

    if(screen->settings->timers_wanted == EINA_TRUE)
    {
  	    screen->timer1 = ecore_timer_del(screen->timer1);
      	screen->timer2 = ecore_timer_del(screen->timer2);
    }
    _navbar_infos_icon_set(screen, STOP_STATUS);

    //Enable exec button again.
    elm_object_disabled_set(screen->settings->exec_bt, EINA_FALSE);

	//Freeing allocated resources.
	screen->_items = items_list_free(screen->_items);	
	screen->items = items_list_free(screen->items);


	ecore_evas_free(screen->ee);
	evas_free(screen->evas);
	free(screen);

    return;
}


static Evas_Object *_get_evas_object_from_edje(Evas_Object *edje, const char *object_name)
{
	Evas_Object *edje_obj;

	edje_obj = edje_object_part_object_get(edje, object_name);
	if(!edje_obj)
	{
		ERR(_("Can't load Edje object from %s!"), object_name);
		evas_object_del(edje_obj);
		return NULL;
	}

	return edje_obj;
}
